@@ -76,8 +76,11 @@ struct _GdictDefboxPrivate
   
   guint show_find : 1;
   guint is_searching : 1;
+  guint is_hovering : 1;
   
   GdkCursor *busy_cursor;
+  GdkCursor *hand_cursor;
+  GdkCursor *regular_cursor;
   
   guint start_id;
   guint end_id;
@@ -102,11 +105,16 @@ enum
   HIDE_FIND,
   FIND_NEXT,
   FIND_PREVIOUS,
+  LINK_CLICKED,
   
   LAST_SIGNAL
 };
 
 static guint gdict_defbox_signals[LAST_SIGNAL] = { 0 };
+static const GdkColor default_link_color = { 0, 0, 0, 0xeeee };
+static const GdkColor default_visited_link_color = { 0, 0x5555, 0x1a1a, 0x8b8b }
+;
+
 
 G_DEFINE_TYPE (GdictDefbox, gdict_defbox, GTK_TYPE_VBOX);
 
@@ -174,6 +182,12 @@ gdict_defbox_finalize (GObject *object)
   
   if (priv->busy_cursor)
     gdk_cursor_unref (priv->busy_cursor);
+
+  if (priv->hand_cursor)
+    gdk_cursor_unref (priv->hand_cursor);
+
+  if (priv->regular_cursor)
+    gdk_cursor_unref (priv->regular_cursor);
   
   G_OBJECT_CLASS (gdict_defbox_parent_class)->finalize (object);
 }
@@ -540,9 +554,10 @@ static void
 gdict_defbox_init_tags (GdictDefbox *defbox)
 {
   GdictDefboxPrivate *priv = defbox->priv;
+  GdkColor *link_color, *visited_link_color;
   
   g_assert (GTK_IS_TEXT_BUFFER (priv->buffer));
-  
+
   gtk_text_buffer_create_tag (priv->buffer, "italic",
   			      "style", PANGO_STYLE_ITALIC,
   			      NULL);
@@ -559,10 +574,36 @@ gdict_defbox_init_tags (GdictDefbox *defbox)
   gtk_text_buffer_create_tag (priv->buffer, "small",
 		  	      "scale", PANGO_SCALE_SMALL,
 			      NULL);  
-  
-  gtk_text_buffer_create_tag (priv->buffer, "emphasis",
-  			      "foreground", "dark green",
-  			      NULL);
+
+  link_color = visited_link_color = NULL;
+  gtk_widget_style_get (GTK_WIDGET (defbox),
+                        "link-color", &link_color,
+                        "visited-link-color", &visited_link_color,
+                        NULL);
+  if (!link_color)
+    link_color = &default_link_color;
+
+  if (!visited_link_color)
+    visited_link_color = &default_visited_link_color;
+
+  gtk_text_buffer_create_tag (priv->buffer, "link",
+                              "underline", PANGO_UNDERLINE_SINGLE,
+                              "foreground-gdk", &link_color,
+                              NULL);
+  gtk_text_buffer_create_tag (priv->buffer, "visited-link",
+                              "underline", PANGO_UNDERLINE_SINGLE,
+                              "foreground-gdk", &visited_link_color,
+                              NULL);
+
+  if (link_color != &default_link_color)
+    gdk_color_free (link_color);
+
+  if (visited_link_color != &default_visited_link_color)
+    gdk_color_free (visited_link_color);
+
+  gtk_text_buffer_create_tag (priv->buffer, "phonetic",
+                              "foreground", "dark gray",
+                              NULL);
 
   gtk_text_buffer_create_tag (priv->buffer, "query-title",
 		              "left-margin", QUERY_MARGIN,
@@ -586,6 +627,185 @@ gdict_defbox_init_tags (GdictDefbox *defbox)
 			      NULL);
 }
 
+static void
+follow_if_is_link (GdictDefbox *defbox,
+                   GtkTextView *text_view,
+                   GtkTextIter *iter)
+{
+  GSList *tags, *l;
+
+  tags = gtk_text_iter_get_tags (iter);
+  
+  for (l = tags; l != NULL; l = l->next)
+    {
+      GtkTextTag *tag = l->data;
+      gchar *name;
+
+      g_object_get (G_OBJECT (tag), "name", &name, NULL);
+      if (name &&
+          (strcmp (name, "link") == 0 ||
+           strcmp (name, "visited-link") == 0))
+        {
+          GtkTextBuffer *buffer = gtk_text_view_get_buffer (text_view);
+          GtkTextIter start, end;
+          gchar *link;
+
+          start = *iter;
+          end = *iter;
+
+          gtk_text_iter_backward_to_tag_toggle (&start, tag);
+          gtk_text_iter_forward_to_tag_toggle (&end, tag);
+
+          link = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
+
+          g_signal_emit (defbox, gdict_defbox_signals[LINK_CLICKED], 0, link);
+
+          g_free (link);
+          g_free (name);
+          
+          break;
+        }
+
+      g_free (name);
+    }
+
+  g_slist_free (tags);
+}
+
+static gboolean
+defbox_event_after_cb (GtkWidget   *text_view,
+                       GdkEvent    *event,
+                       GdictDefbox *defbox)
+{
+  GtkTextIter iter;
+  GtkTextBuffer *buffer;
+  GdkEventButton *button_event;
+  gint bx, by;
+
+  if (event->type != GDK_BUTTON_RELEASE)
+    return FALSE;
+
+  button_event = (GdkEventButton *) event;
+
+  if (button_event->button != 1)
+    return FALSE;
+
+  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
+  if (gtk_text_buffer_get_has_selection (buffer))
+    return FALSE;
+
+  gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (text_view),
+                                         GTK_TEXT_WINDOW_WIDGET,
+                                         button_event->x, button_event->y,
+                                         &bx, &by);
+
+  gtk_text_view_get_iter_at_location (GTK_TEXT_VIEW (text_view),
+                                      &iter,
+                                      bx, by);
+
+  follow_if_is_link (defbox, GTK_TEXT_VIEW (text_view), &iter);
+
+  return FALSE;
+}
+
+static void
+set_cursor_if_appropriate (GdictDefbox *defbox,
+                           GtkTextView *text_view,
+                           gint         x,
+                           gint         y)
+{
+  GdictDefboxPrivate *priv;
+  GSList *tags, *l;
+  GtkTextIter iter;
+  gboolean hovering = FALSE;
+
+  priv = defbox->priv;
+
+  if (!priv->hand_cursor)
+    priv->hand_cursor = gdk_cursor_new (GDK_HAND2);
+
+  if (!priv->regular_cursor)
+    priv->regular_cursor = gdk_cursor_new (GDK_XTERM);
+
+  gtk_text_view_get_iter_at_location (text_view, &iter, x, y);
+
+  tags = gtk_text_iter_get_tags (&iter);
+  for (l = tags; l != NULL; l = l->next)
+    {
+      GtkTextTag *tag = l->data;
+      gchar *name;
+
+      g_object_get (G_OBJECT (tag), "name", &name, NULL);
+      if (name &&
+          (strcmp (name, "link") == 0 ||
+           strcmp (name, "visited-link") == 0))
+        {
+          hovering = TRUE;
+          g_free (name);
+
+          break;
+        }
+
+      g_free (name);
+    }
+
+  if (hovering != defbox->priv->is_hovering)
+    {
+      defbox->priv->is_hovering = hovering;
+
+      if (defbox->priv->is_hovering)
+        gdk_window_set_cursor (gtk_text_view_get_window (text_view,
+                                                         GTK_TEXT_WINDOW_TEXT),
+                               defbox->priv->hand_cursor);
+      else
+        gdk_window_set_cursor (gtk_text_view_get_window (text_view,
+                                                         GTK_TEXT_WINDOW_TEXT),
+                               defbox->priv->regular_cursor);
+    }
+
+  if (tags)
+    g_slist_free (tags);
+}
+
+static gboolean
+defbox_motion_notify_cb (GtkWidget      *text_view,
+                         GdkEventMotion *event,
+                         GdictDefbox    *defbox)
+{
+  gint bx, by;
+
+  gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (text_view),
+                                         GTK_TEXT_WINDOW_WIDGET,
+                                         event->x, event->y,
+                                         &bx, &by);
+
+  set_cursor_if_appropriate (defbox, GTK_TEXT_VIEW (text_view), bx, by);
+
+  gdk_window_get_pointer (text_view->window, NULL, NULL, NULL);
+
+  return FALSE;
+}
+
+static gboolean
+defbox_visibility_notify_cb (GtkWidget          *text_view,
+                             GdkEventVisibility *event,
+                             GdictDefbox        *defbox)
+{
+  gint wx, wy;
+  gint bx, by;
+
+  gdk_window_get_pointer (text_view->window, &wx, &wy, NULL);
+
+  gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (text_view),
+                                         GTK_TEXT_WINDOW_WIDGET,
+                                         wx, wy,
+                                         &bx, &by);
+
+  set_cursor_if_appropriate (defbox, GTK_TEXT_VIEW (text_view), bx, by);
+
+  return FALSE;
+}
+
 static GObject *
 gdict_defbox_constructor (GType                  type,
 			  guint                  n_construct_properties,
@@ -627,6 +847,17 @@ gdict_defbox_constructor (GType                  type,
   priv->find_pane = create_find_pane (defbox);
   gtk_widget_set_composite_name (priv->find_pane, "gdict-defbox-find-pane");
   gtk_box_pack_end (GTK_BOX (defbox), priv->find_pane, FALSE, FALSE, 0);
+
+  /* stuff to make the link machinery work */
+  g_signal_connect (priv->text_view, "event-after",
+                    G_CALLBACK (defbox_event_after_cb),
+                    defbox);
+  g_signal_connect (priv->text_view, "motion-notify-event",
+                    G_CALLBACK (defbox_motion_notify_cb),
+                    defbox);
+  g_signal_connect (priv->text_view, "visibility-notify-event",
+                    G_CALLBACK (defbox_visibility_notify_cb),
+                    defbox);
   
   gtk_widget_pop_composite_child ();
   
@@ -786,6 +1017,15 @@ gdict_defbox_class_init (GdictDefboxClass *klass)
 		  NULL, NULL,
 		  gdict_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
+  gdict_defbox_signals[LINK_CLICKED] =
+    g_signal_new ("link-clicked",
+                  G_OBJECT_CLASS_TYPE (gobject_class),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (GdictDefboxClass, link_clicked),
+                  NULL, NULL,
+                  gdict_marshal_VOID__STRING,
+                  G_TYPE_NONE, 1,
+                  G_TYPE_STRING);
   
   klass->show_find = gdict_defbox_real_show_find;
   klass->hide_find = gdict_defbox_real_hide_find;
@@ -831,13 +1071,15 @@ gdict_defbox_init (GdictDefbox *defbox)
   priv->definitions = NULL;
   
   priv->busy_cursor = NULL;
+  priv->hand_cursor = NULL;
+  priv->regular_cursor = NULL;
   
   priv->show_find = FALSE;
   priv->is_searching = FALSE;
+  priv->is_hovering = FALSE;
   
   priv->tooltips = gtk_tooltips_new ();
-  g_object_ref (priv->tooltips);
-  gtk_object_sink (GTK_OBJECT (priv->tooltips));
+  g_object_ref_sink (G_OBJECT (priv->tooltips));
 }
 
 /**
@@ -1085,13 +1327,68 @@ gdict_defbox_insert_word (GdictDefbox *defbox,
   g_free (text);
 }
 
+/* escape a link string; links are expressed as "{...}".
+ * the link with the '{}' removed is stored inside link_str, while
+ * the returned value is a pointer to what follows the trailing '}'.
+ * link_str is allocated and should be freed.
+ */
+static const gchar *
+escape_link (const gchar  *str,
+             gchar       **link_str)
+{
+  gsize str_len;
+  GString *link_buf;
+  const gchar *p;
+
+  str_len = strlen (str);
+  link_buf = g_string_sized_new (str_len - 2);
+
+  for (p = str + 1; *p != '}'; p++)
+    {
+      link_buf = g_string_append_c (link_buf, *p);
+    }
+
+  if (link_str)
+    *link_str = g_string_free (link_buf, FALSE);
+
+  p++;
+
+  return p;
+}
+
+static const gchar *
+escape_phonethic (const gchar  *str,
+                  gchar       **phon_str)
+{
+  gsize str_len;
+  GString *phon_buf;
+  const gchar *p;
+
+  str_len = strlen (str);
+  phon_buf = g_string_sized_new (str_len - 2);
+
+  for (p = str + 1; *p != '\\'; p++)
+    {
+      phon_buf = g_string_append_c (phon_buf, *p);
+    }
+
+  if (phon_str)
+    *phon_str = g_string_free (phon_buf, FALSE);
+
+  p++;
+
+  return p;
+}
+
 static void
 gdict_defbox_insert_body (GdictDefbox *defbox,
 			  GtkTextIter *iter,
 			  const gchar *body)
 {
   GdictDefboxPrivate *priv;
-  gchar *text;
+  gchar **words;
+  gint len, i;
+  GtkTextIter end_iter;
   
   if (!body)
     return;
@@ -1100,13 +1397,136 @@ gdict_defbox_insert_body (GdictDefbox *defbox,
   priv = defbox->priv;
   
   g_assert (GTK_IS_TEXT_BUFFER (priv->buffer));
-  
-  text = g_strdup_printf ("%s\n", body);
-  
-  gtk_text_buffer_insert (priv->buffer,
-  			  iter,
-  			  text, strlen (text));
-  g_free (text);
+
+  words = g_strsplit (body, " ", -1);
+  len = g_strv_length (words);
+  end_iter = *iter;
+
+  for (i = 0; i < len; i++)
+    {
+      gchar *w = words[i];
+      gint w_len = strlen (w);
+      gchar *begin, *end;
+
+      if (w_len == 0)
+        continue;
+
+      begin = g_utf8_offset_to_pointer (w, 0);
+
+      if (*begin == '{')
+        {
+          end = g_utf8_strrchr (w, -1, '}');
+
+          /* see this is a self contained link */
+          if (end && *end == '}')
+            {
+              const gchar *rest;
+              gchar *link;
+
+              rest = escape_link (w, &link);
+
+              gtk_text_buffer_insert_with_tags_by_name (priv->buffer,
+                                                        &end_iter,
+                                                        link, -1,
+                                                        "link",
+                                                        NULL);
+
+              gtk_text_buffer_insert (priv->buffer, &end_iter, rest, -1);
+
+              gtk_text_buffer_get_end_iter (priv->buffer, &end_iter);
+              gtk_text_buffer_insert (priv->buffer, &end_iter, " ", 1);
+
+              g_free (link);
+
+              continue;
+            }
+          else
+            {
+              /* uh-oh: the link ends in another word */
+              GString *buf;
+              gchar *next;
+              gint cur = i;
+
+              buf = g_string_new (NULL);
+              next = words[cur++];
+
+              while ((end = g_utf8_strrchr (next, -1, '}')) == NULL)
+                {
+                  buf = g_string_append (buf, next);
+                  buf = g_string_append_c (buf, ' ');
+
+                  next = words[cur++];
+                }
+
+              buf = g_string_append (buf, next);
+
+              next = g_string_free (buf, FALSE);
+
+              if (*end == '}')
+                {
+                  const gchar *rest;
+                  gchar *link;
+
+                  rest = escape_link (next, &link);
+
+                  gtk_text_buffer_insert_with_tags_by_name (priv->buffer,
+                                                            &end_iter,
+                                                            link, -1,
+                                                            "link",
+                                                            NULL);
+
+                  gtk_text_buffer_insert (priv->buffer, &end_iter, rest, -1);
+                  gtk_text_buffer_insert (priv->buffer, &end_iter, " ", 1);
+
+                  g_free (link);
+                }
+
+              g_free (next);
+              i = cur;
+
+              continue;
+            }
+        }
+      else if (*begin == '\\')
+        {
+          end = g_utf8_strrchr (w, -1, '\\');
+
+          if (end && *end == '\\')
+            {
+              const gchar *rest;
+              gchar *phon;
+
+              rest = escape_phonethic (w, &phon);
+
+              gtk_text_buffer_insert_with_tags_by_name (priv->buffer,
+                                                        &end_iter,
+                                                        phon, -1,
+                                                        "italic", "phonetic",
+                                                        NULL);
+
+              gtk_text_buffer_insert (priv->buffer, &end_iter, rest, -1);
+
+              gtk_text_buffer_get_end_iter (priv->buffer, &end_iter);
+              gtk_text_buffer_insert (priv->buffer, &end_iter, " ", -1);
+
+              g_free (phon);
+
+              continue;
+            }
+        }
+      
+      gtk_text_buffer_insert (priv->buffer, &end_iter, w, w_len);
+
+      gtk_text_buffer_get_end_iter (priv->buffer, &end_iter);
+      gtk_text_buffer_insert (priv->buffer, &end_iter, " ", 1);
+    }
+
+  gtk_text_buffer_get_end_iter (priv->buffer, &end_iter);
+  gtk_text_buffer_insert (priv->buffer, &end_iter, "\n", 1);
+
+  *iter = end_iter;
+
+  g_strfreev (words);
 }
 
 static void
@@ -1583,3 +2003,34 @@ gdict_defbox_get_font_name (GdictDefbox *defbox)
 
   return defbox->priv->font_name;
 }
+
+/**
+ * gdict_defbox_get_selected_word:
+ * @defbox: a #GdictDefbox
+ *
+ * Since: 0.11
+ */
+gchar *
+gdict_defbox_get_selected_word (GdictDefbox *defbox)
+{
+  GdictDefboxPrivate *priv;
+  GtkTextBuffer *buffer;
+
+  g_return_val_if_fail (GDICT_IS_DEFBOX (defbox), NULL);
+
+  priv = defbox->priv;
+  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (priv->text_view));
+
+  if (!gtk_text_buffer_get_has_selection (buffer))
+    return NULL;
+  else
+    {
+      GtkTextIter start, end;
+      gchar *retval;
+
+      gtk_text_buffer_get_selection_bounds (buffer, &start, &end);
+      retval = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
+
+      return retval;
+    }
+}
