@@ -36,6 +36,8 @@
 void
 gdict_show_about_dialog (GtkWidget *parent)
 {
+  g_return_if_fail (GTK_IS_WIDGET (parent));
+
   const gchar *authors[] = {
     "Mike Hughes <mfh@psilord.com>",
     "Spiros Papadimitriou <spapadim+@cs.cmu.edu>",
@@ -72,7 +74,7 @@ gdict_show_about_dialog (GtkWidget *parent)
     "Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA "
     "02111-1307, USA.\n";
 
-  gtk_show_about_dialog (GTK_WINDOW (parent),
+  gtk_show_about_dialog (GTK_IS_WINDOW (parent) ? GTK_WINDOW (parent) : NULL,
   			 "name", _("Dictionary"),
   			 "version", VERSION,
   			 "copyright", copyright,
@@ -83,5 +85,6 @@ gdict_show_about_dialog (GtkWidget *parent)
   			 "logo-icon-name", "gnome-dictionary",
   			 "license", license,
   			 "wrap-license", TRUE,
+			 "screen", gtk_widget_get_screen (parent),
   			 NULL);
 }
